<?php

namespace Drupal\symfony_translation\EventSubscriber;

use Drupal\Core\DrupalKernelInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Contracts\Translation\TranslatorInterface;

/**
 * Updates the translator language after language negotiation.
 */
class LanguageRequestSubscriber implements EventSubscriberInterface {

  /**
   * The language manager service.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The translation service.
   *
   * @var \Symfony\Component\Translation\Translator
   */
  protected $translation;

  /**
   * Constructs a LanguageRequestSubscriber object.
   *
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager service.
   * @param \Symfony\Component\Translation\Translator $translation
   *   The translation service.
   */
  public function __construct(LanguageManagerInterface $language_manager, TranslatorInterface $translation) {
    $this->languageManager = $language_manager;
    $this->translation = $translation;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = ['onKernelRequestLanguage', 254];
    $events[DrupalKernelInterface::CONTAINER_INITIALIZE_SUBREQUEST_FINISHED][] = ['onContainerInitializeSubrequestFinished', 254];

    return $events;
  }

  /**
   * Initializes the language manager at the beginning of the request.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   The Event to process.
   */
  public function onKernelRequestLanguage(RequestEvent $event) {
    if ($event->isMainRequest()) {
      $this->setLanguageOverrides();
    }
  }

  /**
   * Initializes config overrides whenever the service container is rebuilt.
   */
  public function onContainerInitializeSubrequestFinished() {
    $this->setLanguageOverrides();
  }

  /**
   * Set the default langcode for the string translations.
   */
  private function setLanguageOverrides() {
    $langcode = $this->languageManager->getCurrentLanguage()->getId();
    $this->translation->setLocale($langcode);

    $fallbackCandidates = $this->languageManager->getFallbackCandidates();
    $this->translation->setFallbackLocales($fallbackCandidates);
  }

}
