<?php

namespace Drupal\symfony_translation;

use Drupal\Core\Language\LanguageManagerInterface;
use Symfony\Component\Translation\Loader\JsonFileLoader;
use Symfony\Component\Translation\Loader\PhpFileLoader;
use Symfony\Component\Translation\Loader\YamlFileLoader;
use Symfony\Component\Translation\Translator;
use Symfony\Contracts\Translation\TranslatorInterface;

/**
 * A factory for creating translator instances.
 */
class TranslatorFactory {

  /**
   * The language manager.
   *
   * @var LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Constructs a TranslatorFactory object.
   *
   * @param LanguageManagerInterface $languageManager
   *   The language manager service.
   */
  public function __construct(
    LanguageManagerInterface $languageManager
  ) {
    $this->languageManager = $languageManager;
  }

  /**
   * Creates a translator instance.
   */
  public function getTranslator(): TranslatorInterface {
    $langcode = $this->languageManager->getCurrentLanguage()->getId();

    $translator = new Translator($langcode);
    $translator->setFallbackLocales($this->languageManager->getFallbackCandidates());

    $loaders = [
      'yaml' => new YamlFileLoader(),
      'json' => new JsonFileLoader(),
      'php' => new PhpFileLoader(),
    ];

    foreach ($loaders as $format => $loader) {
      $translator->addLoader($format, $loader);
    }

    $projectTranslationDir = realpath(DRUPAL_ROOT . '/../translations');
    $regex = sprintf('#%s\/((?<domain>.+)(\.|\\|\/))?(?<langcode>.+)\.(?<extension>.+)$#', preg_quote($projectTranslationDir));

    $dirIterator = new \RecursiveDirectoryIterator($projectTranslationDir);
    $iteratorIterator = new \RecursiveIteratorIterator($dirIterator);
    $regexIterator = new \RegexIterator($iteratorIterator, $regex, \RegexIterator::GET_MATCH);

    foreach ($regexIterator as $file) {
      $path = $file[0];
      $domain = $file['domain'] ?? 'messages';
      $langcode = $file['langcode'];

      if ($file['extension'] === 'yml') {
        $format = 'yaml';
      }
      else {
        $format = $file['format'];
      }

      if (!isset($langcode) || !isset($format)) {
        continue;
      }

      $translator->addResource($format, $path, $langcode, $domain);
    }

    return $translator;
  }

}
